// Base abstract interface for COM along with definitions for supporting
// constants and data types.  This module replicates the bare essentials 
// of what is found in <windows.h> for use with text-only or numeric-only 
// COM programs and DLLs that don't have <windows.h> as a dependency.
// Programs having #include <windows.h> don't need this module.
//
// COM -- Component Object Model -- is the convention for object interfaces 
// accessible across program module boundaries under Microsoft Windows.
//
// COM is based on C++, and C++ implements object interfaces as abstract
// base classes -- classes comprised of virtual, abstract functions.  An
// abstract function is a function definition where the implementation is
// deferred to a derived class -- note the "= 0" notation in C++ after the
// function definition.
//
// IUnknown is the root interface that all COM objects support -- more
// complex interfaces can extend IUnknown by inheritance and can add
// more virtual abstract functions.  The IUnknown interface
// implements QueryInterface(), for discovery of the other interfaces that
// are extensions of IUnknown supported by an object, and AddRef() and
// Release() support reference-counted lifetime management of that object.

#ifndef COMBaseH
#define COMBaseH

//#define NULL 0
#define S_OK 0
#define E_NOINTERFACE 0x80004002
#define HRESULT unsigned long
#define ULONG unsigned long

#ifdef _WIN32
#define STDCALL __stdcall
#define LIBEXPORT extern "C" __declspec(dllexport)
#else
#define STDCALL
#define LIBEXPORT extern "C"
#endif

struct _GUID
{
    unsigned long  d1;
    unsigned short d2;
    unsigned short d3;
    unsigned char  d4[8];
};

struct IUnknown
{
    virtual HRESULT STDCALL QueryInterface(const struct _GUID& guid, void** pvObj) = 0; 
    virtual ULONG STDCALL AddRef() = 0; 
    virtual ULONG STDCALL Release() = 0; 
};

#endif
