// Copyright 2004 Paul Milenkovic, 118 Shiloh Dr., Madison, WI 53705 U.S.A.

#ifndef FilterFactoryH
#define FilterFactoryH

#include <string>
#include "WaveCpp.h"
#include "WaveBase.h"

// Base class for implementing interface IFilterFactory.  
//
// GetFilter() and GetDialog() are given default implementations that return
// NULL interfaces -- if a derived class supplies a filter but does not supply
// a dialog, it can override GetFilter() and leave the default implementation
// of GetDialog as-is.
//
// Think of GetNamed() as an application-specific counterpart to QueryInterface()
// where you can supply a meaningful object name instead of a GUID.
// The default implementation of GetNamed() supports "WaveFilterC" to invoke
// GetFilter(), "FilterDialog" to invoke GetDialog.   GetNamed() may be
// overriden in future implementations to support the creation of objects
// with other types of interfaces.

class FilterFactory : public IFilterFactory
{
public:
    virtual void STDCALL SetStr(char* str) {}

    virtual bool STDCALL GetFilter(IWaveFilterC** wave_filter)
    {
        *wave_filter = NULL;
        return false;
    }

    virtual bool STDCALL GetDialog(IFilterDialog** filter_dlg)
    {
        *filter_dlg = NULL;
        return false;
    }

    virtual bool STDCALL GetNamed(char* name, IUnknown** named)
    {
        if (string(name) == "WaveFilterC")
            return GetFilter((IWaveFilterC**) named);
        else
        if (string(name) == "FilterDialog")
            return GetDialog((IFilterDialog**) named);
        else
        {
            *named = NULL;
            return false;
        }
    }

    FilterFactory() {}
};

#endif
