// Copyright 2004 Paul Milenkovic, 118 Shiloh Dr., Madison, WI 53705 U.S.A.
// Free software: Permission granted to use, copy, excerpt or share this
// source code and derived executables provided that this notice is maintained.  

// Base class for zero-phase non-causal FIR filters of length N = 2M+1.
//
// Override protected method CalcCoef() to compute filter coefficients:
// T is the sample interval in seconds.  Default filter is a length N
// boxcar function

import net.cspeech.DSP.*;

public class FIRFilter extends NonCausal implements SISOFilter {

    private float[] fcoef; 
    private int M = 0; // fcoef.length == 2*M+1
    
    // Enables echo to text console in Java app
    public void setConsoleOut(long consoleH) {
        setConsoleOut(new ConsoleOut(consoleH));
    }
   
    protected void CalcCoef(float[] fcoef, int M, double T) {        
        for (int k = -M; k <= M; k++)
            fcoef[M-k] = (float) (0.54 + 0.46*Math.cos(Math.PI*k/M))/fcoef.length;
    }
    
    protected void Init(double sample_ms) {
        print("FIRFilter.Init "); super.Init(sample_ms);
        setM_pre(M); setM_pst(M);
        fcoef = new float[2*M+1];
        CalcCoef(fcoef, M, 0.001*sample_ms);
    }
		
    protected void Apply(float[] src, float[] dst) {
        for (int n = 0; n < dst.length; n++) {
            float sum = 0.0f; int Mmk = 0;           
            while (Mmk < fcoef.length) {
            	sum += src[n+Mmk]*fcoef[Mmk]; Mmk++;}                
            dst[n] = sum;
        }
    }

    public void setParams(double[] params) {
        if (params.length > 0) M = (int) params[0];
    }
}
