// Copyright 2012 Paul Milenkovic, 118 Shiloh Dr., Madison, WI 53705 U.S.A.
// Free software: Permission granted to use, copy, excerpt or share this
// source code and derived executables provided that this notice is maintained.  

import net.cspeech.DSP.*;

public class FMDemodulator extends EnvelopeFilter implements SISOFilter {
    
    private double f_c, K_f, smi1, smq1, wr, wi, K_dphii;
    private int M = 100;

    protected void Init(double sample_ms) {
        super.Init(sample_ms);
        double T = 0.001*sample_ms;
        wr = Math.cos(2*Math.PI*f_c*T); wi = Math.sin(2*Math.PI*f_c*T);
        K_dphii = 1.0/(2.0*Math.PI*K_f*T);
        smi1 = 0.0; smq1 = 0.0;
    }
		
    protected void Apply(float[] src, float[] dst) {
        for (int n = 0; n < dst.length; n++) {
            
        // In-phase and quadrature FM signal    
            double smi = iSamp(src, n), smq = qSamp(src, n); // z[n] = smi + j smq
            
         // ejp[n] = z[n] * z^*[n-1]
            double epr = smi*smi1 + smq*smq1, epi = -smi*smq1 + smq*smi1; 
         // ejd[n] = ejp[n] * (wr - j wi)  removes carrier phase increment   
            double dvr = epr*wr+epi*wi, dvi = -epr*wi+epi*wr;  
            double m_n = Math.atan2(dvi, dvr)*K_dphii;
       
            dst[n] = (float) m_n;
            //dst[n] = (float) smi; // This is your Re(z(t))
            //dst[n] = (float) ((-phr*wi+phi*wr)*K_dphii);
            smi1 = smi; smq1 = smq;
        }
    }

    public void setParams(double[] params) {
    	super.setParams(new double[] {M});
        if (params != null) {
            if (params.length > 0) f_c = params[0];
            if (params.length > 2) K_f = params[2];
        }
    }
}