// Copyright 2011, 2020 Paul Milenkovic, 118 Shiloh Dr., Madison, WI 53705 U.S.A.
// Free software: Permission granted to use, copy, excerpt or share this
// source code and derived executables provided that this notice is maintained.  

import net.cspeech.DSP.*;

public class FMLoopC extends EnvelopeFilter implements SISOFilter {
    
    private double Kf = 2000.0, f_loop = 500, msg, rho = 1;
    private int M = 100;
    private FMModulator vco = new FMModulator();
    private LP1st loop_filt = new LP1st();

    public void setConsoleOut(long consoleH) {
        setConsoleOut(new ConsoleOut(consoleH));
    }

    protected void Init(double sample_ms) {
        super.Init(sample_ms);
        vco.setMs(sample_ms);
        double T = 0.001*sample_ms;
        double theta = Math.sqrt(2.0)*Math.PI*f_loop*T, gamma = 1.0 - rho*theta,
               gamma2 = gamma*gamma,
               d0 = 1. - 2.*gamma*Math.cos(theta) + gamma2,
               b1 = 1 - gamma2 - 0*d0, // multiply d0 by 1/G open loop inverse gain
               b0 = b1 + d0,
               a1 = gamma2 + b1;
        b1 /= 2.0*Math.PI*Kf*T; b0 /= 2.0*Math.PI*Kf*T;
        msg = 0;
        loop_filt.setParams(new double[] {a1, b0, b1});
        loop_filt.setMs(sample_ms);

    }
		
    protected void Apply(float[] src, float[] dst) {
        for (int n = 0; n < dst.length; n++) {
            
        // In-phase and quadrature FM signal    
            double smi = iSamp(src, n), smq = qSamp(src, n);
     
            vco.vUpd(msg);
        // Complex-valued VCO output and phase detector
            double mc = smi*vco.vRe() + smq*vco.vIm(),
                   ms = smq*vco.vRe() - smi*vco.vIm();
            
        // Real-valued loop filter    
            msg = loop_filt.gety0(Math.atan2(ms, mc));
            dst[n] = (float) (msg);
        }
    }

    public void setParams(double[] params) {
    	super.setParams(new double[] {M});
        if (params != null) {
            vco.setParams(params);
            if (params.length > 2) Kf = params[2];
            if (params.length > 4) f_loop = params[4];
            if (params.length > 5) rho = params[5];
        }
    }
}
