// Copyright 2011 Paul Milenkovic, 118 Shiloh Dr., Madison, WI 53705 U.S.A.
// Free software: Permission granted to use, copy, excerpt or share this
// source code and derived executables provided that this notice is maintained.  

import net.cspeech.DSP.*;

public class FMModulator extends InPlace implements SISOFilter {    
    
    private double f_c = 5000.0, A_c = 1.0, K_f = 0.1, n_v = 0.0;
    private double wr, wi, vr, vi, K_dphiv2;
    private int ndbl = 1;

    protected void Init(double sample_ms) {
        super.Init(sample_ms);
        K_dphiv2 = Math.PI*K_f*0.001*sample_ms;
        for (int i = 0; i < ndbl; i++) K_dphiv2 = 0.5*K_dphiv2;
        // Calculate coefficients from filter parameters
        wr = Math.cos(2*Math.PI*f_c*0.001*sample_ms);
        wi = Math.sin(2*Math.PI*f_c*0.001*sample_ms);
        // Initialize state
        vr = A_c; vi = 0.0;
    }

    public final void vUpd(double f_d) {
        double dphiv2 = K_dphiv2*f_d, dphiv2sq = dphiv2*dphiv2;
        // Small angle approximation tan(dphi/2) = dphi/2,
        // formulas for cos(dphi), sin(dphi) from tan(dphi/2).
        double c = (1.0 - dphiv2sq)/(1.0 + dphiv2sq), 
               s = 2.0*dphiv2/(1.0 + dphiv2sq);
        for (int i = 0; i < ndbl; i++) { // frequency doubling
            double temp = c*c - s*s; s = c*s + s*c; c = temp;           
        } 
        c = Math.cos(4*K_dphiv2*f_d); s = Math.sin(4*K_dphiv2*f_d);
        // Apply frequency deviation to carrier phase delta
        double wrr = wr*c - wi*s, wii = wr*s + wi*c;
        // Apply total phase delta to output waveform    
        double temp = vr*wrr - vi *wii; vi = vi*wrr + vr*wii; vr = temp;
    }
    
    public final double vRe() {return vr;}

    public final double vIm() {return vi;}

    protected void Apply(float[] sig) {
        for (int i = 0; i < sig.length; i++) {
            vUpd(sig[i]);
            double z0 = Math.sqrt(-2*Math.log(Math.random()))*
            		       Math.cos(2*Math.PI*Math.random());
            //sig[i] = (float) (vr + z0*n_v/Math.sqrt(12));
            sig[i] = (float) (vr + n_v*(Math.random()-0.5));
        }
    }

    public void setParams(double[] params) {
        if (params != null) {
            if (params.length > 0) f_c = params[0];
            if (params.length > 1) A_c = params[1];
            if (params.length > 2) K_f = params[2];
            if (params.length > 3) n_v = params[3]*Math.sqrt(12.0);
        }
    }
}
