// Copyright 2011 Paul Milenkovic, 118 Shiloh Dr., Madison, WI 53705 U.S.A.
// Free software: Permission granted to use, copy, excerpt or share this
// source code and derived executables provided that this notice is maintained.  

import net.cspeech.DSP.*;

public class QuadHybrid extends NonCausal implements SISOFilter {
    
    private int M = 100;
    private float[] fcoef;
    private double phase = -90, f_m = 0.0, f_c = 0.0;

    protected void Init(double sample_ms) {
        super.Init(sample_ms);
        setM_pre(M); setM_pst(M); 
        double PI = Math.PI, h_n, rphase = PI*phase/180.0, T = .001*sample_ms;
        fcoef = new float[2*M+1];
        if (f_m == 0.0) f_m = 0.25/T;
        if (f_c == 0.0) f_c = f_m-1.0/(T*M);
        for (int n = -M; n <= M; n++) {
            if (n == 0) h_n = 2*f_c*T;
                   else h_n = Math.sin(2*PI*f_c*n*T)/(PI*n);
            h_n = h_n*(0.54 + 0.46*Math.cos((PI*n)/M));
            fcoef[n+M] = (float) (h_n*2.0*Math.cos(2.0*PI*f_m*n*T+rphase));
        }
    }
		
    protected double outputSamp(float[] src, int n) {
    	int Nm1 = fcoef.length-1; double sum = 0.0; 
        for (int k = 0; k <= Nm1; k++) sum += fcoef[k]*src[n+Nm1-k]; 
        return sum;
    }
    
    protected void Apply(float[] src, float[] dst) {
        for (int n = 0; n < dst.length; n++) 
            dst[n] = (float) outputSamp(src, n);
    }

    public void setParams(double[] params) {
        if (params.length > 0) phase = params[0];
        if (params.length > 1) M = (int) params[1];
        if (params.length > 2) f_m = params[2];
        if (params.length > 3) f_c = params[3];
    }
}