// Copyright 2007 Paul Milenkovic, 118 Shiloh Dr., Madison, WI 53705 U.S.A.
// Free software: Permission granted to use, copy, excerpt or share this
// source code and derived executables provided that this notice is maintained.  

import net.cspeech.DSP.*;

public class SineFilter extends InPlace implements SISOFilter {
	
    private double freq  = 100.0, mag = 1.0, phase = 0.0;
    // internal coeffcients, state
    private double dphase, sphase;
    private NumericOut num_out = new NumericOut();
    private double G_re, G_im, W_re, W_im;  

    // Enables echo to text console in Java app
    public void setConsoleOut(long consoleH) {
        setConsoleOut(new ConsoleOut(consoleH));
    }
    
    protected void Init(double sample_ms) {
        num_out.print("SineFilter.Init "); num_out.print();
        num_out.println(sample_ms); 
        super.Init(sample_ms);
     // Calculate coefficients from filter parameters
     // Initialize state
        dphase = 2.0*Math.PI*freq*0.001*sample_ms;
        sphase = 2.0*Math.PI*phase/360.0;
     // Calculate real and imaginary parts of initial G = W_0, of W below  
    }

    public final double sineVal() {
     // Replace direct trig evals with Eq. 1 for new sineVal, Eq. 2 for update of G 	
    	double sval = mag*Math.cos(sphase);
    	sphase += dphase;
    	if (sphase > Math.PI) sphase -= 2.*Math.PI;
        return sval;
    }
    
    protected void Apply(float[] sig) {
        for (int n = 0; n < sig.length; n++) {
            // Output data point
            sig[n] += (float) (sineVal());
        }
    }

    public void setParams(double[] params) {
        if (params != null) {
            if (params.length > 0) freq  = params[0];
            if (params.length > 1) mag   = params[1];
            if (params.length > 2) phase = params[2];
        }
    }
}
