// Copyright 2004 Paul Milenkovic, 118 Shiloh Dr., Madison, WI 53705 U.S.A.

#ifndef AddTrackH
#define AddTrackH

#include <string>
#include "SISOFilter.h"
#include "NumStream.h"
#include "WaveCpp.h"

#define max_mem  4096
#define max_buf  8192

class AddTrack : public SISOFilter 
{
private:
    INumStreamC* num_stream;

    float x[max_buf];
    int ninp, npos;

protected:
    virtual  int Get_mem() {return 0;}

    virtual void Apply(float sig_buf[], int nbuf, int npos) {}

    virtual void SrcSet()
    {
        SISOFilter::SrcSet();
        ninp = 0;
        npos = 0;
    }

    string SetExt(string fname, string ext)
    {
    	size_t epos = fname.rfind('.');
        if (epos != string::npos)
            fname = fname.substr(0,epos);
        return fname + ext;    
    }

    string SetFname(string pname, string fname)
    {
    	size_t epos = pname.rfind('\\');
        if (epos != string::npos)
            pname = pname.substr(0,epos+1);
        return pname + fname;    
    }

	bool Open(string ename) 
    {
        if (num_stream != NULL)
            return num_stream->Open((char *) ename.c_str());
        else {
		    
            Set_num_stream(new NumStream());
            IWaveDescriptorC* dsc;
            SISOFilter::GetDescriptor(&dsc);
            string fname = SetExt(dsc->Get_fname(),ename);
            dsc->Release();
            return num_stream->Open((char *) fname.c_str());
        }
    }

    void Close() {num_stream->Close();}

    bool GetLine() {return num_stream->GetLine();}

    bool GetNum(double& val) {return num_stream->GetDouble(val);}

    int  GetCoef(float* coef, int mcoef)
    {
        int ncoef = 0;
        while (ncoef < mcoef && num_stream->GetSingle(coef[ncoef]))
            ncoef++;
        return ncoef;
    }

public:
    virtual bool STDCALL EndOfSrc()
    {
        return (ninp <= 0 && SISOFilter::EndOfSrc());
    }

    void  STDCALL TransmitNow(float* wave_samp, int nBuf, int& nSamp)
    {
        if (!SISOFilter::EndOfSrc()) {

            int new_ninp = nBuf + Get_mem();
            if (new_ninp > max_buf)
                new_ninp = max_buf;

            int nTN;
            SISOFilter::TransmitNow(x+ninp,new_ninp-ninp,nTN);

            if (nTN > 0) {
                ninp += nTN;
                Apply(x,ninp,npos);
            }
        }

        int nout = ninp;
        if (!SISOFilter::EndOfSrc())
            nout -= Get_mem();
        if (nout > nBuf)
            nout = nBuf;

        if (nout > 0) {

            int i;
            for (i = 0; i < nout; i++)
                wave_samp[i] = x[i];
            for (i = 0; i < ninp - nout; i++)
                x[i] = x[i+nout];
            ninp -= nout;
            npos += nout;
            nSamp = nout;
        } else
            nSamp = 0;
    }

    virtual void  STDCALL Dispose() 
    {
        if (num_stream != NULL)
            num_stream->Close();
        SISOFilter::Dispose();
    }

    virtual void  STDCALL Set_num_stream(INumStreamC* num_stream) 
    {
        if (this->num_stream != NULL)
            this->num_stream->Release();
        num_stream->AddRef();
        this->num_stream = num_stream;
    }

    AddTrack() : num_stream(NULL) {}

    ~AddTrack() 
    {
        if (num_stream != NULL) {

            num_stream->Close();
            num_stream->Release();
            num_stream = NULL;
        }
    }

    virtual ULONG STDCALL Release() 
    {
        int rc = decRefCount();
        if (rc <= 0) delete this;
        return rc;
    }
};

#endif
