// Copyright 2007 Paul Milenkovic, 118 Shiloh Dr., Madison, WI 53705 U.S.A.

#ifndef BeamPelH
#define BeamPelH

#include "AddTrack.h"

class BeamPel : public AddTrack 
{
private:
    double sample_ms;

    bool valid_epoch;
    double t_epoch;
    int i_epoch;

    double pel_val;
	double npel;

    bool GetEpoch()
    {
        bool result = GetLine();
		t_epoch += 6.866;
        i_epoch = int(t_epoch/sample_ms);
        return result;
    }

    bool GetParams()
    {
		double pel_inc;
		int i = 0;
		while (i < npel && GetNum(pel_inc)) i++;
		if (i < npel)
			return false;
		else {
			pel_val += pel_inc;
			return true;
		}
    }

    void PutPulse(float* sig_buf, int ipulse, int fpulse, int nbuf)
    {
		if (ipulse < 0) ipulse = 0;
		if (fpulse > nbuf) fpulse = nbuf;
		for (int i = ipulse; i < fpulse; i++)
			if (pel_val >= 1000000.0)
				sig_buf[i] = 0;
			else
                sig_buf[i] = (float) (pel_val/1000.0);
    }

protected:
    int Get_mem() {return 4096;}

    void Init(IWaveDescriptorC* dsc)
    {
        SISOFilter::Init(dsc);
        valid_epoch = Open(SetExt(dsc->Get_fname(),".xyd")) && GetEpoch();
    }

    void Init(double sample_ms)
    {
        this->sample_ms = sample_ms;
		t_epoch = -6.866*1.5;
		pel_val = 0.0;
    }

    void Apply(float sig_buf[], int nbuf, int npos)
    {
		if (i_epoch < npos)
			i_epoch = npos;
		for (int i = i_epoch-npos; i < nbuf; i++)
			sig_buf[i] = 0.0f;

		while (valid_epoch && i_epoch-npos < nbuf - 1024) 
        {
            if (GetParams()) {
				int prev_i_epoch = i_epoch;
                valid_epoch = GetEpoch();
                PutPulse(sig_buf,prev_i_epoch-npos,i_epoch-npos,nbuf);
			} else 
                valid_epoch = false;
		}
    }

public:
	BeamPel() : npel(6) {}

	virtual void  STDCALL GetDescriptor(IWaveDescriptorC** dsc) 
    {
		AddTrack::GetDescriptor(dsc);
		(*dsc)->Set_uname("mm");
		(*dsc)->Set_range(100.0);
    }

	virtual void  STDCALL Set_param_list(double* param_list, int nParam) 
	{
		if (nParam > 0)
            npel = (int) param_list[0];
	}
};

#endif
