// Copyright 2004 Paul Milenkovic, 118 Shiloh Dr., Madison, WI 53705 U.S.A.

#ifndef FIRFilterH
#define FIRFilterH
#include "NonCausal.h"
#include "math.h"

class FIRFilter : public NonCausal
{
private:
    float fcoef[4097];
    int M, ncoef;

protected:
    virtual void CalcCoef(float* fcoef, int ncoef, double T)
    {
        int N = ncoef;
    	double dc_scale = N*0.54 - 0.46;
    	dc_scale = 1.0/dc_scale;
        for (int i = 0; i < N; i++)
            fcoef[i] = (float) (dc_scale*(0.54 - 0.46*cos(2.0*M_PI*i/(N-1.0))));;
    }

    virtual void Init(IWaveDescriptorC* dsc)
    {
        SISOFilter::Init(dsc);
    }

    virtual void Init(double sample_ms)
    {
        setM_pre(M);
        setM_pst(M);
        ncoef = 2*M + 1;
        CalcCoef(fcoef,ncoef,0.001*sample_ms);
    }

    virtual void Apply(float* src, float* dst, int nSamp) 
    {
        CheckInTime();
        for (int i = 0; i < nSamp; i++)
        {
            float sum = 0.0f; int j = 0;
	        while (j < ncoef) {
                sum += src[i+j]*fcoef[j];
		        j++;
	        }
            dst[i] = sum;
        }
        CheckOutTime(nSamp*ncoef,(char*) "Avg MACC (ns) =",EndOfSrc() != 0);
    }

public:
    FIRFilter() {}

    virtual void  STDCALL Set_param_list(double* param_list, int nParam) 
    {
        if (nParam > 0)	
        {
            M = (int) param_list[0];
            if (M > 2048)  M = 2048;
        }
    }
};

#endif
