// Copyright 2011 Paul Milenkovic, 118 Shiloh Dr., Madison, WI 53705 U.S.A.

#ifndef FIRNotchH
#define FIRNotchH
#include "NonCausal.h"
#include "FIRFilter.h"
#include "math.h"

class FIRNotch : public FIRFilter
{
private:
    double fc, tr;
    
protected:
    virtual void CalcCoef(float* fcoef, int ncoef, double T)
    {
        int N = ncoef; int M = N/2; double ff; 
        DoubleOut(2.0*M_PI*fc*T,10,6); OutLn();
        for (int n = -M; n <= M; n++) {
            ff = cos(2.0*M_PI*fc*T*n) 
                 + cos(2.0*M_PI*(fc*T-1.0/N)*n)
                 + cos(2.0*M_PI*(fc*T+1.0/N)*n)
                 + cos(2.0*M_PI*(fc*T-2.0/N)*n)
                 + cos(2.0*M_PI*(fc*T+2.0/N)*n)
                 + tr*cos(2.0*M_PI*(fc*T-3.0/N)*n)
                 + tr*cos(2.0*M_PI*(fc*T+3.0/N)*n);
            if (n == 0)    
                fcoef[n+M] = (float) (1.0 - 2.0*ff/N);
            else
                fcoef[n+M] = (float) (-2.0*ff/N);
            
        }
    }

public:
    FIRNotch() : fc(2500.0), tr(0.5) {}

    virtual void  STDCALL Set_param_list(double* param_list, int nParam) 
    {
    	FIRFilter::Set_param_list(param_list, nParam);
        if (nParam > 1)	
        	fc = param_list[1];
        if (nParam > 2)	
        	tr = param_list[2];
    }
};

#endif
