// Copyright 2004 Paul Milenkovic, 118 Shiloh Dr., Madison, WI 53705 U.S.A.

#ifndef FricSynH
#define FricSynH

#include <cstdlib>
#include <math.h>
#include "LPCObj.h"
#include "AddTrack.h"

class FricSyn : public AddTrack 
{
private:
    double sample_ms;

    bool valid_epoch;
    double t_epoch;
    int i_epoch;

    int mfric;
    double fric_amp;
    LPCObj LPC_obj;

    bool GetEpoch()
    {
        bool result = GetLine() && GetNum(t_epoch);
        if (result)
            i_epoch = int(t_epoch/sample_ms);
        return result;
    }

    bool GetParams()
    {
        bool result = GetNum(fric_amp);
        if (result)
        {
            float la[257];
            int ncoef = GetCoef(la+1,256);
            LPC_obj.SetLog2Area(la,ncoef);
        }
        return result;
    }


    void GenPulse(float* pulse, int npulse, int mfric, 
        double fric_amp, double sample_ms)
    {
    // Design deemphasis filter
        float a2 = (float) exp(-M_PI*0.100*sample_ms);
        float a1 = (float) (-2.0*cos(2.0*M_PI*0.050*sample_ms)*a2);
		a2 = a2*a2;

        int nrand = 2*mfric + 1;
        if (nrand > npulse) 
            nrand = npulse;

		float amp = (float) (sqrt(12.0)*fric_amp/RAND_MAX);
        int i;
        for (i = 0; i < nrand; i++)
            pulse[i] = (float) (amp*sin(M_PI*i/(nrand-1))*(rand() - 0.5*RAND_MAX));
        for (i = nrand; i < npulse; i++)
            pulse[i] = 0.0;

    // Apply deemphasis filter
		float x1 = 0;
		float y1 = 0;
		float y2 = 0;
        for (i = 0; i < npulse; i++) {
			float x0 = pulse[i];
			float y0 = x0 - x1 - a1*y1 - a2*y2;
			y2 = y1;
			y1 = y0;
			x1 = x0;
			pulse[i] = y0;
		}
    }

    void PutPulse(float* sig_buf, int ipulse)
    {
        if (ipulse >= mfric)
        {
            float pulse_buf[512+1024];
            int npulse = mfric + int(20.0/sample_ms);
            if (npulse > mfric + 1024)
                npulse = mfric + 1024;

            GenPulse(pulse_buf,npulse,mfric,fric_amp,sample_ms);
            LPC_obj.ForwardFilter(pulse_buf,pulse_buf,npulse,npulse);

            float dc_gain = LPC_obj.DCGain();
        // Output pulse
            float* sig_init = sig_buf+ipulse-mfric;
            for (int i = 0; i < npulse; i++)
                sig_init[i] += dc_gain*pulse_buf[i];
        }
    }

protected:
    int Get_mem() {return 4096;}

	void Init(IWaveDescriptorC* dsc)
	{
		SISOFilter::Init(dsc);
	}

	void Init(double sample_ms) 
    {
        this->sample_ms = sample_ms;
        mfric = (int) (5.0/sample_ms);
        valid_epoch = Open(".fla") && GetEpoch();
    }

    void Apply(float sig_buf[], int nbuf, int npos)
    {
        while (valid_epoch && i_epoch < npos + nbuf - 1024) {
            if (GetParams()) {
                if (i_epoch-npos >= 0)
                    PutPulse(sig_buf,i_epoch-npos);
                valid_epoch = GetEpoch();
            } else
                valid_epoch = false;
        }
    }
};

#endif
