// Copyright 2004 Paul Milenkovic, 118 Shiloh Dr., Madison, WI 53705 U.S.A.

#ifndef GlotSynH
#define GlotSynH

#include <math.h>
#include "LPCObj.h"
#include "AddTrack.h"

class GlotSyn : public AddTrack 
{
private:
    double sample_ms;

    bool valid_epoch;
    double t_epoch;
    int i_epoch;

    double glot_amp, glot_freq, glot_bw;
    LPCObj LPC_obj;

    bool GetEpoch()
    {
        bool result = GetLine() && GetNum(t_epoch);
        if (result)
            i_epoch = int(t_epoch/sample_ms);
        return result;
    }

    bool GetParams()
    {
        bool result = GetNum(glot_amp) && GetNum(glot_freq) && GetNum(glot_bw);
        if (result)
        {
            float la[257];
            int ncoef = GetCoef(la+1,256);
            LPC_obj.SetLog2Area(la,ncoef);
        }
        return result;
    }

    void GenPulse(float* buf, int nglot, 
        double glot_amp, double glot_freq, double glot_bw, double sample_ms)
    {
    // Design pulse-shaping filter   
        float a2 = (float) exp(-M_PI*glot_bw*sample_ms);
        float a1 = (float) (-2.0*cos(2.0*M_PI*glot_freq*sample_ms)*a2);
        a2 = a2 * a2;
    // Apply pulse-shaping filter
        buf[nglot-0] = (float) (-glot_amp/a1);
        buf[nglot-1] = (float) glot_amp;
        buf[nglot-2] = (float) (glot_amp*(1.0/a1 - a1 + a2/a1));
        for (int i = nglot-3; i >= 0; i--) 
            buf[i] = -a1*buf[i+1] - a2*buf[i+2];
    }

    void PutPulse(float* sig_buf, int ipulse, int lpulse)
    {
    // Apply pulse-shaping filter
        int nglot = 512;
        if (nglot > ipulse) 
            nglot = ipulse;
        if (nglot > 2*lpulse)
            nglot = 2*lpulse;
        float pulse_buf[512+1024];
        GenPulse(pulse_buf+512-nglot,nglot,glot_amp,glot_freq,glot_bw,sample_ms);
    // Apply vocal tract filter
        LPC_obj.ForwardFilterD(
            pulse_buf+512-nglot,
            pulse_buf+512-nglot,
            nglot+1,nglot+1024,lpulse);
    // Output pulse
        float dc_gain = LPC_obj.DCGain();
        for (int i = -nglot; i < 1024; i++)
            sig_buf[ipulse+i] += dc_gain*pulse_buf[512+i];
    }

protected:
    int Get_mem() {return 4096;}

	void Init(IWaveDescriptorC* dsc)
	{
		SISOFilter::Init(dsc);
	}

	void Init(double sample_ms)
	{
        this->sample_ms = sample_ms;
        valid_epoch = Open(".gpl") && GetEpoch();
    }

    void Apply(float sig_buf[], int nbuf, int npos)
    {
        while (valid_epoch && i_epoch < npos + nbuf - 1024) {
            if (GetParams()) {
				int ip = i_epoch - npos;
				valid_epoch = GetEpoch();
				int ipp;
				if (valid_epoch)
					ipp = i_epoch - npos;
                else
				    ipp = nbuf;
                if (ip >= 0)
                    PutPulse(sig_buf,ip,ipp-ip);
            } else
                valid_epoch = false;
        }
    }
};

#endif
