// Copyright 2004 Paul Milenkovic, 118 Shiloh Dr., Madison, WI 53705 U.S.A.

#ifndef LPCObjH
#define LPCObjH

#include <math.h>
#include "WaveCpp.h"
#include "NumStream.h"

class LPCObj 
{ 
private:
    float pcoef[257];
    int ncoef;

    void Log2AreatoLPC(float* a, int ncoef)
    {
        int i, j;
        double u, aj;

        a[0] = 1.0;
        for (i = 1; i <= ncoef; i++) {
            u = exp(a[i]*log(2.0));
            u = (u - 1.0)/(u + 1.0);
            for (j = 1; j <= i/2; j++) {
                aj     = a[j]   + u*a[i-j];
                a[i-j] = (float) (a[i-j] + u*a[j]);
                a[j]   = (float) aj;
            }
            a[i] = (float) u;
        }
    }

public:
    void SetLog2Area(float* la, int ncoef)
    {
        if (ncoef > 256)
            ncoef = 256;
        this->ncoef = ncoef;
        pcoef[0] = 0.0;
        for (int i = 1; i <= ncoef; i++)
            pcoef[i] = la[i];
        Log2AreatoLPC(pcoef,ncoef);
    }

    void ForwardFilter(float* x, float* y, int nx, int ny)
    {
        if (nx > ny) nx = ny;
        int isig;

        for (isig = 0; isig < nx; isig++) {

            int nmem = ncoef;
            if (nmem > isig) nmem = isig;
            float sum = x[isig];
            for (int j = 1; j <= nmem; j++)
                sum -= pcoef[j]*x[isig-j];
            y[isig] = sum;
        }

        for (isig = nx; isig < ny; isig++) {

            int nmem = ncoef;
            if (nmem > isig) nmem = isig;
            float sum = 0;
            for (int j = 1; j <= nmem; j++)
                sum -= pcoef[j]*x[isig-j];
            y[isig] = sum;
        }
    }

    void ForwardFilterD(float* x, float* y, int nx, int ny, int lp)
    // with pulse-synchronous damping -- lp is the pulse length
    {
        float dcoef[257];
        for (int k = 0; k <= ncoef; k++)
            dcoef[k] = pcoef[k];

        if (nx > ny) nx = ny;
        int isig;

        for (isig = 0; isig < nx; isig++) {

            int nmem = ncoef;
            if (nmem > isig) nmem = isig;
            float sum = x[isig];
            for (int j = 1; j <= nmem; j++)
                sum -= dcoef[j]*x[isig-j];
            y[isig] = sum;
        }

        for (isig = nx; isig < ny; isig++) {

            int nmem = ncoef;
            if (nmem > isig) nmem = isig;

            if (isig - nx > 0.6*lp && isig - nx <= 0.8*lp) {

                float dmp = 1.0;
                for (int k = 1; k <= ncoef; k++) {
                    dmp = (float) (dmp * 0.9985);
                    dcoef[k] = dcoef[k] * dmp;
                }
            }

            float sum = 0;
            if (isig - nx < 1.5*lp)
                for (int j = 1; j <= nmem; j++)
                    sum -= dcoef[j]*x[isig-j];
            y[isig] = sum;
        }
    }

    float DCGain()
    {
        float dc_gain = 0;
        for (int j = 0; j <= ncoef; j++)
            dc_gain += pcoef[j];
        return dc_gain;
    }

    LPCObj() : ncoef(0) {}
};

#endif
