// Copyright 2004 Paul Milenkovic, 118 Shiloh Dr., Madison, WI 53705 U.S.A.

#ifndef NumStreamH
#define NumStreamH

#include <iostream>
#include <fstream>
#include <sstream>
#include <string>

using namespace std;

class NumStream : public INumStreamC
{
private:
    ifstream num_file;
    istringstream* num_line;
	bool is_bin;
    string line_val;
	int bytes_remain;

    void Delete_num_line()
    {
		if (num_line != NULL)
			delete num_line;
		num_line = NULL;
    }

public:
    virtual bool   STDCALL Open(char* str)
    {
        Close();
		if (is_bin) {
			num_file.open(str,ios::in | ios::binary | ios::ate);
			if (num_file.is_open()) {
				bytes_remain = (int) (num_file.tellg());
				num_file.seekg(0,ios::beg);
			}
		} else
		    num_file.open(str,ios::in);
        return num_file.is_open();
    }

	virtual bool Open(string s) {return Open((char*) s.c_str());}

	virtual bool OpenFromString(string s) {return Open(s);}

    virtual void   STDCALL Close()
    {
		Delete_num_line();
        if (num_file.is_open())
            num_file.close();
		bytes_remain = 0;
    }

	virtual int BytesRemain() 
	{
		if (is_bin && num_file.is_open()) 
			return bytes_remain;
		else
			return 0;
	}

	virtual void SkipBytes(int bytes_skip)
	{
		if (is_bin && num_file.is_open()) {
			if (bytes_skip > bytes_remain) bytes_skip = bytes_remain;
			num_file.seekg(bytes_skip,ios::cur);
		    bytes_remain -= bytes_skip;
		}
	}

	virtual int ReadBytes(char buf[], int bytes_read)
	{
		if (is_bin && num_file.is_open()) {
		    if (bytes_read > bytes_remain) bytes_read = bytes_remain;
            num_file.read(buf,bytes_read);
		} else
		    bytes_read = 0;
		return bytes_read;
	}

    virtual bool   STDCALL GetLine()
    {
		Delete_num_line();
        if (!is_bin && num_file.is_open() && getline(num_file,line_val)) 
            num_line = new istringstream(line_val);
        return num_line != NULL; 
    }

    virtual bool   STDCALL GetDouble(double& val)
    {
        return num_line != NULL && *num_line >> val;
    }

    virtual bool   STDCALL GetSingle(float& val)
    {
        return num_line != NULL && *num_line >> val;
    }

    NumStream() : num_line(NULL), is_bin(false) {}

	NumStream(bool is_bin) : num_line(NULL), is_bin(is_bin), bytes_remain(0) {}

    ~NumStream() {Close();}

    virtual ULONG STDCALL Release() 
    {
		int rc = decRefCount();
		if (rc <= 0) delete this;
		return rc;
    }
};

#endif