// Copyright 2004 Paul Milenkovic, 118 Shiloh Dr., Madison, WI 53705 U.S.A.

#ifndef ResampleBlackmanH
#define ResampleBlackmanH

#include "Resample.h"
#include <math.h>

class ResampleBlackman : public Resample
{
private:
	double F_c, F_b, D_c;
protected:
	virtual void CalcCoef(float* fcoef, int& ncoef)
	{
        ncoef = getnUp()*(getM_pst() + getM_pre()) + 1;
		if (ncoef == 1)
		    fcoef[0] = 1.0;
		else
		{
            double rmid = (ncoef - 1.0)/2;
            for (int k = 0; k < ncoef; k++)
            {
			    // Ideal LPF, f_c = (F_c * D_c/rmid)*nUp*f_s/2.
                if (k == rmid)
                    fcoef[k] = (float) (F_c + D_c/rmid);
                else
                    fcoef[k] = (float) (sin(M_PI*(k-rmid)*(F_c + D_c/rmid))/(M_PI*(k-rmid)));
			    // Modulate to center frequency f_b = F_b*nUp*f_s/2, 
			    // apply Blackman window.
                fcoef[k] = (float) (fcoef[k]*getnUp()*
				    cos(M_PI*(k-rmid)*F_b)*
				    (0.42 - 0.5*cos(M_PI*k/rmid) + 0.08*cos(2*M_PI*k/rmid)));
            }
		}
	}
public:
	ResampleBlackman() : Resample(), F_b(0.0), D_c(0.52) {}

	virtual void  STDCALL Set_param_list(double* param_list, int nParam) 
    {
		Resample::Set_param_list(param_list,nParam);
		// F_b		Fractional band center frequency: f_b = F_b*nUp*f_s/2
		//				F_b = 0 lowpass, = 1 highpass, 0 < F_b < 1 bandpass, 
		// M_c		ncoef = 2*(nUp*(M_c/nUp)) + 1,
		// F_c		Fractional LPF corner frequency: f_c = F_c*nUp*f_s/2
		// D_c		Fractional LPF corner frequency displacement.

		int M_c = 60;
        if (getnUp() > getnDn())
            F_c = 1.0/getnUp();
		else
			F_c = 1.0/getnDn();

		if (nParam > 4) F_b = param_list[4];
		if (nParam > 5) M_c = (int) param_list[5];
		if (nParam > 6) F_c = param_list[6];
        if (nParam > 7) D_c = param_list[7];

		if (M_c > 2048)	M_c = 2048;
        setM_pre(M_c/getnUp());
        setM_pst(M_c/getnUp());
    }
};

#endif