// Copyright 2004 Paul Milenkovic, 118 Shiloh Dr., Madison, WI 53705 U.S.A.

#ifndef ResamplePSOLAH
#define ResamplePSOLAH

#include "SISOFilter.h"
#include <math.h>

class ResamplePSOLA : public SISOFilter
{
private:
    int M_pre, M_pst; // pre: look-behind; pst: look-ahead.
    int nUp, nDn; 
	int	nsrc, nend;
	bool end_of_src;
	float src_buf[8192];
    float fcoef[4097];
	int ncoef;

protected:
	void setM_pre(int M_pre) {this->M_pre = M_pre;}
	void setM_pst(int M_pst) {this->M_pst = M_pst;}
	int  getM_pre() {return M_pre;}
	int  getM_pst() {return M_pst;}
	int  getnUp() {return nUp;}
	int  getnDn() {return nDn;}

	virtual void CalcCoef(float* fcoef, int& ncoef)
	{
		ncoef = 1;
		fcoef[0] = 1.0;
	}

	virtual void SrcSet() 
	{
		SISOFilter::SrcSet();

		CalcCoef(fcoef,ncoef);

		nsrc = M_pre;
		for (int i = 0; i < nsrc; i++)
			src_buf[i] = 0;

		nend = M_pst;
		end_of_src = false;
	}

	virtual void Apply(float* src, float* dst, int mSrc, int mDst, int& nSrc, int& nDst) 
	{
        nDst = (mSrc*nUp)/nDn;
		if (nDst > mDst) nDst = mDst;
		nSrc = (nDst*nDn)/nUp;

		for (int i = 0; i < nDst && i < nSrc; i++)
			dst[i] = src[i];
	}

public:
	ResamplePSOLA() : nDn(2), nUp(1), M_pre(0), M_pst(0), end_of_src(true)
    {
		Set_param_list(NULL,0);
    }

	virtual void  STDCALL Set_param_list(double* param_list, int nParam) 
    {
		if (nParam > 0) nDn = (int) param_list[0];
        if (nParam > 1) nUp = (int) param_list[1];
    }

	virtual bool STDCALL EndOfSrc() 
	{
		return end_of_src;
	}

	virtual void  STDCALL GetDescriptor(IWaveDescriptorC** dsc) 
	{
        SISOFilter::GetDescriptor(dsc);
        (*dsc)->Set_msamp(((*dsc)->Get_msamp()*nUp)/nDn);
	}

	virtual void  STDCALL TransmitNow(float* wave_samp, int nBuf, int& nRet)
	{	
        int nSamp = (nBuf*nDn + nUp - 1)/nUp;
		if (nSamp > 8192-M_pre-M_pst)
			nSamp = 8192-M_pre-M_pst;

        int nReq = nSamp + M_pre + M_pst - nsrc;
        if (!SISOFilter::EndOfSrc() && nReq > 0)
        {
            int nTransmit;
            SISOFilter::TransmitNow(src_buf+nsrc,nReq,nTransmit);
            nsrc += nTransmit;
            nReq -= nTransmit;
        }
        if (SISOFilter::EndOfSrc() && nReq > 0)
        {
            if (!end_of_src)
            {
                if (nReq > nend)
                    nReq = nend;
                nend -= nReq;
            }
            for (int i = nsrc; i < nsrc + nReq; i++)
                src_buf[i] = 0.0;
            nsrc += nReq;
        }
            
		nRet = 0;
		if (nsrc - M_pre - M_pst > 0)
		{
            int nAdv;
		    Apply(src_buf,wave_samp,nsrc-M_pre-M_pst,nBuf,nAdv,nRet);

		    // Shift src_buf.
			for (int i = 0; i < nsrc - nAdv; i++)
				src_buf[i] = src_buf[nAdv+i];
			nsrc -= nAdv;
		}

		end_of_src = nend <= 0 && nsrc - M_pre - M_pst <= 0 && SISOFilter::EndOfSrc();
	}
};

#endif