// Copyright 2004 Paul Milenkovic, 118 Shiloh Dr., Madison, WI 53705 U.S.A.

#ifndef SineFilterH
#define SineFilterH

#include "InPlace.h"
#include <math.h>

class SineFilter : public InPlace
{
private:
// filter parameters
    double freq_Hz, magnitude, phase;
// filter coefficients w, internal state y
    double wr, wi, yr, yi;

protected:
    virtual void Init(IWaveDescriptorC* dsc)
    {
        SISOFilter::Init(dsc);
    }

    virtual void Init(double sample_ms)
    {
    // Calculate coefficients from filter parameters
        double iphase = 2*M_PI*phase/360;
        double dphase = 2*M_PI*freq_Hz*0.001*sample_ms;
        wr = cos(dphase);
        wi = sin(dphase);
    // Initialize state
        yr = magnitude*cos(iphase);
        yi = magnitude*sin(iphase);
    }

    virtual void Apply(float* sig, int nSig)
    {
        CheckInTime();
        for (int i = 0; i < nSig; i++) {
            sig[i] += float(yr);
            // Calculate next sine function value
            double yrr = yr;
            yr = yr*wr -  yi*wi;
            yi = yi*wr + yrr*wi;
        }
        CheckOutTime(nSig,"complex multiply (ns) =",EndOfSrc() != 0);
    }

public:
    SineFilter() : freq_Hz(100), magnitude(1.0), phase(0.0) {}

    SineFilter(double freq, double mag, double phase) :
        freq_Hz(freq), magnitude(mag), phase(phase) 
    {}

    virtual void  STDCALL Set_param_list(double* param_list, int nParam) 
    {
        if (nParam > 0) freq_Hz   = param_list[0];
        if (nParam > 1) magnitude = param_list[1];
        if (nParam > 2) phase     = param_list[2];
    }
};

#endif
